#include "tunnels_player.hpp"
#include "utils/godot_macros.h"
#include "utils/player_input.hpp"
#include <godot_cpp/classes/input_event.hpp>
#include <godot_cpp/classes/viewport.hpp>

namespace godot {
float const TunnelsPlayer::MOVE_SPEED{0.f};

void TunnelsPlayer::_bind_methods() {
#define CLASSNAME TunnelsPlayer
    GDFUNCTION_ARGS(horizontal_move_input, "event", "value");
    GDFUNCTION_ARGS(vertical_move_input, "event", "value");
}

void TunnelsPlayer::_ready() { GDGAMEONLY();
    this->camera = this->get_viewport()->get_camera_3d();
}

void TunnelsPlayer::_physics_process(double delta_time) { GDGAMEONLY();
    this->set_velocity(this->get_world_move_input());
    this->move_and_slide();
}

void TunnelsPlayer::setup_player_input(PlayerInput *input) {
    input->listen_to(PlayerInput::Listener("move_right", "move_left", this, "horizontal_move_input"));
    input->listen_to(PlayerInput::Listener("move_forward", "move_backward", this, "vertical_move_input"));
}

Node *TunnelsPlayer::to_node() {
    return Object::cast_to<Node>(this);
}

void TunnelsPlayer::horizontal_move_input(Ref<InputEvent> event, float value) {
    this->move_input.x = value;
}

void TunnelsPlayer::vertical_move_input(Ref<InputEvent> event, float value) {
    this->move_input.y = value;
}

Vector3 TunnelsPlayer::get_world_move_input() const {
    Basis const camera_basis = camera->get_global_transform().get_basis();
    return this->move_input.x * camera_basis.get_column(0)
         + this->move_input.y * camera_basis.get_column(2);
}
}
